/**
 * \class filegenerator
 * \author  eac/spg/gvm
 * \version 1.0
 *
 * \section LICENSE
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details at
 * http://www.gnu.org/copyleft/gpl.html
 */


#ifndef _FILEGENERATOR_H
#define	_FILEGENERATOR_H

#include <TROOT.h>
#include <fstream>
#include <iostream>
#include <string>
#include <iomanip>
#include <cstdlib>
#include <TROOT.h>
using namespace std;

/** \brief A file manipulation class.
This class reads and write SiPM's characterization data from/to a file.
*/
class filegenerator {
public:
	/**
	Assigns the file data.file to be read/written by the object.
	*/
    filegenerator();
	/**
	Assigns the custom fileName to be read/written by the object.
	\param fileName is the physical file name to be read/written.
	*/
    filegenerator(const string & fileName);
	/**
	Opens a file to write into.
	\return TRUE is a sucessful operation, FALSE means failure
	*/
    bool openW();
	/**
	Opens a file to read from.
	\return TRUE is a sucessful operation, FALSE means failure
	*/
    bool openR();
	/**
	Writes to a file in write mode.
	\param id Is the SiPM's identifier
	\param zeroValue Is the SiPM's lower voltage value.
	\param spanValue Is the SiPM's upper voltage value.
	\return TRUE is a sucessful operation, FALSE means failure
	*/
    bool writeFile(const Int_t & id, const Float_t & zeroValue, const Float_t & spanValue);
	/**
	Reads from a file in reading only mode.
	\param id Is the SiPM's identifier
	\param zeroValue Is the SiPM's lower voltage value.
	\param spanValue Is the SiPM's upper voltage value.
	\return TRUE is a sucessful operation, FALSE means failure
	*/
    bool readFile(Int_t & id, Float_t & zeroValue, Float_t & spanValue);
	/**
	Closes any opened files.
	*/
    virtual ~filegenerator();
private:
	string m_sFileName;
	ofstream m_fWrite;
	ifstream m_fRead;
};

#endif	/* _FILEGENERATOR_H */

