/**
 * \class hardware
 *
 * \author $Author: ne0/ac/akis $
 *
 * \version $Revision: 1.0 $
 *
 * \date $Date: $
 *
 * Contact: ealvarado@elo.utfsm.cl, spizarro@elo.utfsm.cl, gvalinm@elo.utfsm.cl
 *
 */

#ifndef _HARDWARE_H
#define	_HARDWARE_H

#include "voltagesource.h"
#include "heatsource.h"
#include "photomultiplier.h"	
#include "sensor.h"
#include "sipm.h"

/**
* \brief This class implements the real idea of the hardware to be used in the actual experiment. It associates with another classes to give a realistic approach.
*/
class hardware {
public:
		///Class void constructor.
    hardware();
		///Class copy constructor.
    hardware(const hardware& orig);
		///Class destructor.
    virtual ~hardware();
		///Class constructor to connect VoltageSource, HeatSource and Sipm.
    hardware(VoltageSource &_Voltage, HeatSource &_Temperature, sipm &_Sipm);
		///Method to set the VoltageSource if needed.
    void setVoltageSource(VoltageSource &_Voltage);
		///Method to set the HeatSource if needed.
    void setTemperature(HeatSource &_Temperature);
		///Method to connect Sipm if needed.
    void connectSipm(sipm &_Sipm);
		///Method to connect Sensor if needed.
		void connectSensor(sensor &_Sensor);
		///Method to measure current of the hardware (sipm).
    virtual Float_t getMeasureCurrent();
		///Method to measure voltage of the hardware (voltagesource).
    Float_t getMeasureVoltage();
		///Method to measure temperature of the hardware (heatsource).
    Float_t getTemperature();
		///Method to get the output of this hardware.
    virtual Float_t getOutput();
		///Method to get the ID of this hardware
    Int_t getID();
		void setID(Int_t i);
    void setNewTemp(Float_t f);
private: 
    int MyID;
    VoltageSource Voltage;
    HeatSource Temperature;
    sipm Sipm;
		sensor Sensor;
};

#endif	/* _HARDWARE_H */

