/* 
 * File:   lightguide.cpp
 * Author: ac
 * Simulador Guia de Luz
 */
#include "lightguide.h"


lightguide::lightguide(Int_t simulate3D, Int_t _numberOfEvents){
	if(simulate3D>0) 
		SHOW_3D=kTRUE;
	else
		SHOW_3D=kFALSE;
	DEBUG = kFALSE;
	SHOW_NORMALS = kFALSE;
	SHOW_INTERSECTION = kFALSE;

	numberOfEvents = _numberOfEvents;
}

void lightguide::run(){


  TVector3 ab;
  TVector3 d, od;
  Double_t dmag;
  TVector3 rc;
  Double_t min;
  Double_t t;
  TVector3 o;
  Double_t s;
  Double_t in, out;
  TVector3 vin, vout;
  Double_t pPhi, sqrDist, dPhi, cosTheta;
  TVector3 pos;
  TVector3 dir;
  TVector3 outPos;
  Double_t outAngle;  
  Int_t nlost=0; 
  TApplication theApp("App", 0, 0);



  const Int_t N_EVENTS = numberOfEvents;
  // Crear Cilindro
  TVector3 CYL_A, CYL_B;
  Double_t CYL_R;
  CYL_A.SetXYZ(0.0,0.0,0.0);
  CYL_B.SetXYZ(0.0,0.0,10.0);
  CYL_R = 1.0;
  ab = (CYL_B - CYL_A).Unit();

 // Medio 1, Medio 2
 // Ley de Snell o Snel

  Double_t MED_N = 1.0;
//  Double_t CYL_N = 2.0;
  Double_t CYL_N = 1.490;
  Double_t TIR_A = MED_N/CYL_N;

  // Crear Canvas para root
  TCanvas canvas("clg","Cilindrical Light Guide", 800, 600);
  canvas.Connect("Closed()", "TApplication", &theApp, "Terminate()");
  canvas.Divide(2,2);
  gStyle->SetPalette(1);
  
 // Histograma posicion rayos entrantes.
  TH2D posHist("eph","Position of entering rays", 120, -1.1*CYL_R, 1.1*CYL_R, 120, -1.1*CYL_R, 1.1*CYL_R);
 // Histograma direccion rayos.
  TH3D dirHist("edh","Direction of entering rays", 120, -1.1, 1.1, 120, -1.1, 1.1, 120, -1.1, 1.1);
 // Histograma del coseno del angulo de salida.
  TH1D outDirHist("odr","Number of rays per exiting angle cosine", 120, -0.1, 1.1);
 // Histograma posicion rayos salientes.
  TH2D outPosHist("opr","Position of exiting rays", 120, -1.1*CYL_R, 1.1*CYL_R, 120, -1.1*CYL_R, 1.1*CYL_R);
	


  if (gGeoManager) {
    delete gGeoManager;
  }
  TGeoManager *geom = new TGeoManager("world","Physical World For Lightguide");
  if (SHOW_3D) {
    TEveManager::Create();
  }
  
  TGeoMaterial vac("Vacuum",0,0,0);
  TGeoMaterial al("Al",26.98,13,2.7);
  TGeoMedium med("VacuumMed", 1,&vac);
  TGeoMedium alMed("AlMed", 2, &al);

  TGeoVolume *world = geom->MakeBox("World",&med,5.0,5.0,5.0);
  geom->SetTopVolume(world);


  Double_t midHeight = (CYL_B - CYL_A).Mag()/2.0;
  TGeoTube *cylinder = new TGeoTube("cylinder", 0.0, CYL_R, midHeight);
  TGeoVolume *lg = new TGeoVolume("light_guide",cylinder,&alMed);
  lg->SetLineColor(kBlue);
  TGeoTranslation t1(0.0,0.0,midHeight);
  world->AddNode(lg,1,&t1);

  gRandom->SetSeed();

  for (Int_t i=0; i<N_EVENTS; i++) {

	// Generado de eventos: Posicion
    sqrDist = gRandom->Uniform(0.0,CYL_R);
    pPhi = gRandom->Uniform(0, TMath::Pi()*2.0) ;
    pos.SetX(TMath::Sqrt(sqrDist)*TMath::Cos(pPhi));
    pos.SetY(TMath::Sqrt(sqrDist)*TMath::Sin(pPhi));
    pos.SetZ(0.0);
    posHist.Fill(pos.X(), pos.Y());

	// Generador de eventos: Angulo

    cosTheta = gRandom->Uniform(0.0,1.0);
    dPhi = gRandom->Uniform(0, TMath::Pi()*2.0);
    dir.SetX(TMath::Sin(TMath::ACos(cosTheta))*TMath::Cos(dPhi));
    dir.SetY(TMath::Sin(TMath::ACos(cosTheta))*TMath::Sin(dPhi));
    dir.SetZ(cosTheta);
    dirHist.Fill(dir.X(), dir.Y(), dir.Z());

    while (kTRUE) {

      d = dir.Cross(ab);
      rc = pos - CYL_A;
	if ((dir.Cross(ab)).Mag2()==0) {
        min = (rc-(rc.Dot(ab)*ab)).Mag();
        if (min>CYL_R) {
          nlost++;
          break;
        }
      } else {
        od = d.Unit();
        dmag = d.Mag();
        min = TMath::Abs(rc.Dot(od));
        if (min>CYL_R) {
          nlost++;
          break;
        }
        t = ((rc.Cross(ab)).Dot(od))/dmag;
        o = (od.Cross(ab)).Unit();
        s = TMath::Abs( TMath::Sqrt((CYL_R*CYL_R) - (min*min)) / (dir.Dot(o)) );
        in = -t - s;
        out = -t + s;
        vin = pos + in*dir;
        vout = pos + out*dir;

//Normal
        TVector3 hb = CYL_A - vout;
        TVector3 n = (hb - (hb.Dot(ab)*ab)).Unit();
        if (SHOW_NORMALS) {
          TEveArrow *normal = new TEveArrow(n.X(), n.Y(), n.Z(), vout.X(), vout.Y(), vout.Z());
          normal->SetMainColor(kRed); 
          normal->SetTubeR(0.001);
          normal->SetConeR(0.0);
          gEve->AddElement(normal);
        }


        TGeoVolume *bout;
        if (SHOW_INTERSECTION) {
          TGeoTranslation *trout = new TGeoTranslation(vout.X(), vout.Y(), vout.Z());
          TGeoVolume *bout = gGeoManager->MakeBox("bout", &alMed, .02, .02, .02);
          bout->SetLineColor(kGreen);
          world->AddNode(bout,3, trout);
        }
        TEveArrow *track; 
        if (SHOW_3D) {
          track = new TEveArrow(vout.X()-pos.X(), vout.Y()-pos.Y(), vout.Z()-pos.Z(), pos.X(), pos.Y(), pos.Z());
          track->SetMainColor(kGreen);
          track->SetTubeR(0.001);
          track->SetConeR(0.001);
          gEve->AddElement(track);
        }
        TVector3 N = ab.Unit();
        Double_t D = CYL_B.Mag();

        Double_t capDir = dir.Dot(N);
        Double_t rayCap = (-pos.Dot(N) + D)/capDir;

        if (rayCap <= out) {
          outPos = pos+(rayCap*dir);
          outAngle = TMath::ACos(capDir);
          outPosHist.Fill(outPos.X(), outPos.Y());
          outDirHist.Fill(capDir);
          break;
        }

	// Ley de snel
        Double_t dot = n.Dot(dir);
        if (TMath::Sin(TMath::ACos(dot))<TIR_A) {
          if (SHOW_INTERSECTION) {
            bout->SetLineColor(kRed);
          }
          if (SHOW_3D) {
            track->SetMainColor(kRed);
          }
          break;
        }
// rayo que rebota
        pos = vout;
        dir = dir - (2 * dot * n);
      }
    }
  }

  gGeoManager->CloseGeometry();
  geom->SetTopVisible();
  TGeoNode *geoTop = geom->GetTopNode();
  if (SHOW_3D) {
    TEveGeoTopNode *eveTop = new TEveGeoTopNode(geom, geoTop);
    eveTop->SetVisLevel(4);
    eveTop->GetNode()->GetVolume()->SetVisibility(kFALSE);

    gEve->AddGlobalElement(eveTop);
    gEve->Redraw3D(kTRUE);
  }
//Dibujamos histogramas
  canvas.cd(1);
  posHist.Draw("SURF4");

  canvas.cd(3);
  dirHist.Draw();

  canvas.cd(2);
  outDirHist.Draw();

  canvas.cd(4);
  outPosHist.Draw();

  canvas.Modified();
  canvas.Update();
  theApp.Run();

}


lightguide::~lightguide(){}

