/**
 * \class sipm
 *
 * This class is meant to be as real as possible for an good approximation to the sensor to be used on real experimentation. It implements methods to give some random errors 
 * associated to the fact that each sipm it's not identical to the next.
 *
 * \author $Author: ne0/ac/akis $
 *
 * \version $Revision: 1.0 $
 *
 * \date $Date: $
 *
 * Contact: ealvarado@elo.utfsm.cl, spizarro@elo.utfsm.cl, gvalinm@elo.utfsm.cl
 *
 */
#ifndef _SIPM_H
#define	_SIPM_H
#include "photomultiplier.h"
#include "sensor.h"
#include <TROOT.h>
#include "TRandom1.h"
/**
 * \brief This class inherits from class photomultiplier. It uses it's redefined methods to get a different current / voltage characteristic.
 */
class sipm:public Photomultiplier {
public:
	///Constructor
    sipm();
	///Copy Constructor
    sipm(const sipm& orig);
	///Inits SiPM with connections
    sipm(Float_t _MagneticAngle, Float_t _MagneticMagnitude, VoltageSource  &_InputVoltage, HeatSource  &_InputTemperature, Float_t _BreakdownVoltage);
	///Gets current depending on external parameters
    Float_t getMeasureCurrent(Float_t voltage, Float_t temp);
	///Gets actual voltage
    Float_t getMeasureVoltage();
	///Get Break Down Voltage - Operating voltage
    Float_t getBreakdownVoltage();
	///Gets Random1
    Float_t setSeed();
	///Gets Random2
    Float_t setSeed2();
	///Sets Current Voltage Source
    void setVsource(VoltageSource &_InputVoltage);
    virtual ~sipm();
private:
    VoltageSource InputVoltage;
    HeatSource InputTemperature;
    Float_t Seed;
    Float_t SeedUnico;
    Float_t DarkCurrent;
    Float_t Gain_db;
    Float_t MagneticAngle;
    Float_t MagneticMagnitude; 
    Float_t p1;
    Float_t p2;
};

#endif	/* _SIPM_H */

